require 'fileutils'
require 'json'
require_relative 'aws_mfa/config_loader'
require_relative 'aws_mfa/credentials_output_executor'
require_relative 'aws_mfa/credentials_loader'
require_relative 'aws_mfa/errors'
require_relative 'aws_mfa/profile_config'
require_relative 'aws_mfa/shell_command'

class AwsMfa
  MYDRIVE_CREDENTIALS_FILE_PATH = '.mydrive-aws'

  def initialize
    verify_aws_installed
    verify_config_file
    @mydrive_credentials_cache_dir = set_mydrive_credentials_cache_dir
  end

  def execute(execution_output = :set_env, profile = nil)
    return if ENV["AWS_SESSION_TOKEN"]

    aws_profile = profile || ENV['AWS_PROFILE'] || "default"
    profile_config = load_profile_config(aws_profile)
    credentials = load_credentials(profile_config)
    execute_output(execution_output, credentials)
  end

  private

  attr_reader :mydrive_credentials_cache_dir

  def verify_aws_installed
    unless AwsMfa::ShellCommand.new("which aws").call.success?
      raise Errors::CommandNotFound, "Could not find the aws command"
    end
  end

  def verify_config_file
    aws_config
  end

  def set_mydrive_credentials_cache_dir
    credential_directory = File.join(ENV['HOME'], MYDRIVE_CREDENTIALS_FILE_PATH)
    FileUtils.mkdir_p(credential_directory) unless File.directory?(credential_directory)
    credential_directory
  end

  def load_profile_config(profile)
    if profile_config = aws_config[profile]
      AwsMfa::ProfileConfig.new(profile_config, profile)
    else
      raise Errors::Error, "MFA serial number not found for profile '#{profile}'"
    end
  end

  def load_credentials(profile_config)
    CredentialsLoader.new(
      mydrive_credentials_cache_dir,
      expiration_buffer_minutes: expiration_buffer_minutes
    ).load_credentials(profile_config)
  end

  def execute_output(execution_output, credentials)
    CredentialsOutputExecutor.new.execute_output(execution_output, credentials)
  end

  def expiration_buffer_minutes
    ENV.fetch("MYDRIVE_MFA_TOKEN_EXPIRATION_BUFFER", 0).to_i
  end

  def aws_config
    @aws_config ||= ConfigLoader.build.load
  end
end
